/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "vreg2.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define PWM_PERIOD_ERROR   0

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void vreg2_cfg_setup ( vreg2_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->pwm = HAL_PIN_NC;
    
    // Additional gpio pins

    cfg->an   = HAL_PIN_NC;

	cfg->dev_pwm_base 	  = MODULE_TIM_1;
	cfg->dev_pwm_channel  = TIM_CHANNEL_NC;
	cfg->dev_pwm_polarity = TIM_ACTIVE_LOW;
	cfg->dev_pwm_freq 	  = VREG2_DEF_FREQ;
}

VREG2_RETVAL vreg2_init ( vreg2_t *ctx, vreg2_cfg_t *cfg )
{
    pwm_config_t pwm_cfg;
    
    pwm_configure_default( &pwm_cfg );

	pwm_cfg.pin      = cfg->pwm;
	pwm_cfg.base     = cfg->dev_pwm_base;
	pwm_cfg.channel  = cfg->dev_pwm_channel;
	pwm_cfg.polarity = cfg->dev_pwm_polarity;
	pwm_cfg.freq_hz  = cfg->dev_pwm_freq;

    ctx->pwm_freq = cfg->dev_pwm_freq;
    ctx->pwm_period = pwm_open( &ctx->pwm, &pwm_cfg );

    if ( ctx->pwm_period == PWM_PERIOD_ERROR )
    {
        return VREG2_INIT_ERROR;
    }

    // Input pins

    digital_in_init( &ctx->an, cfg->an );

    return VREG2_OK;

}

void vreg2_set_duty_cycle ( vreg2_t *ctx, pwm_data_t duty_cycle )
{
    pwm_set_duty( &ctx->pwm, duty_cycle ); 
}

void vreg2_pwm_stop ( vreg2_t *ctx )
{
    pwm_stop( &ctx->pwm ); 
}

void vreg2_pwm_start ( vreg2_t *ctx )
{
    pwm_start( &ctx->pwm ); 
}

// ------------------------------------------------------------------------- END
