/*!
 * \file 
 * \brief Vreg2 Click example
 * 
 * # Description
 * This application enables voltage regulation.
 *
 * The demo application is composed of two sections :
 * 
 * ## Application Init 
 * Initializes the GPIO driver and configures the PWM 
 * peripheral for controlling VREG2 voltage output.
 * 
 * ## Application Task  
 * Based on the data received from the uart the voltage output will be increased or decreased.
 * 
 * 
 * \author MikroE Team
 *
 */
// ------------------------------------------------------------------- INCLUDES

#include "board.h"
#include "log.h"
#include "vreg2.h"

// ------------------------------------------------------------------ VARIABLES

static vreg2_t vreg2;
static log_t logger;

static pwm_data_t duty_cycle = 100;
static uint8_t ctrl_flag = 0;
static uint8_t ctrl_pre_flag = 0;

// ------------------------------------------------------ APPLICATION FUNCTIONS

void application_init ( void )
{
    log_cfg_t log_cfg;
    vreg2_cfg_t cfg;

    //  Logger initialization.

    log_cfg.level = LOG_LEVEL_DEBUG;
    log_cfg.rx_pin = (0x27) ; log_cfg.tx_pin = (0x26) ; log_cfg.baud = 9600; log_cfg.level = LOG_LEVEL_DEBUG; ;
    log_init( &logger, &log_cfg );
    log_info( &logger, "---- Application Init ----" );

    //  Click initialization.

    vreg2_cfg_setup( &cfg );
    cfg.pwm = (0x3C) ; cfg.an = (0x03) ;
    vreg2_init( &vreg2, &cfg );

    vreg2_set_duty_cycle( &vreg2, duty_cycle );
    vreg2_pwm_start( &vreg2 );
}

void application_task ( void )
{
    //  Task implementation.

    switch ( ctrl_flag )
    {
        case 0:
        {
            duty_cycle = (uint32_t)vreg2.pwm_period  * 20 / 100;
            log_printf( &logger, "Volatage set to : %d %%\r\n", duty_cycle );
            ctrl_pre_flag = ctrl_flag;
            ctrl_flag = 1;
            break;
        }

        case 1:
        {
            duty_cycle = (uint32_t)vreg2.pwm_period * 50 / 100;
            log_printf( &logger, "Volatage set to : %d %%\r\n", duty_cycle );

            if ( ctrl_pre_flag == 0 )
            {
                ctrl_pre_flag = ctrl_flag;
                ctrl_flag = 2;
            }
            else if ( ctrl_pre_flag == 2 )
            {
                ctrl_pre_flag = ctrl_flag;
                ctrl_flag = 0;
            }
            break;
        }
        case 2:
        {
            duty_cycle = (uint32_t)vreg2.pwm_period  * 70 / 100;
            log_printf( &logger, "Volatage set to : %d %%\r\n", duty_cycle );

            if ( ctrl_pre_flag == 1 )
            {
                ctrl_pre_flag = ctrl_flag;
                ctrl_flag = 3;
            }
            else if ( ctrl_pre_flag == 3 )
            {
                ctrl_pre_flag = ctrl_flag;
                ctrl_flag = 1;
            }
            break;
        }
        case 3:
        {
            duty_cycle = (uint32_t)vreg2.pwm_period * 95 / 100;
            log_printf( &logger, "Volatage set to : %d %%\r\n", duty_cycle );
            ctrl_pre_flag = ctrl_flag;
            ctrl_flag = 2;
            break;

        }
        default:
        {
            log_printf( &logger, "Something broke\r\n");
        }
    }

    vreg2_set_duty_cycle( &vreg2, duty_cycle );
    Delay_ms( 1500 );
    Delay_100ms();
}

void main ( void )
{
    application_init( );

    for ( ; ; )
    {
        application_task( );
    }
}

// ------------------------------------------------------------------------ END
